/* Cautare Binara (Binary Search)
 *
 * Este o metoda rapida de a cauta o valoare intr-un sir ORDONAT
 * 
 * Cautari:
 * 1. Cautare secventiala (de la poz 1 la n) 
 * 		=> Complexitate O(n)
 * 2. Cautare Binara
 * 		=> Complexitate: O(log2(n))
 * 
 * Ex: n = 17, cautam v = 42
 * 
 * valori:  1 3 5 7 12 24 29 35 41  57  80 100 150 200 260
 * pozitii: 1 2 3 4  5  6 7  8  9   10  11  12  13  14  15
 *          st                                          dr
 *                                                      sn
 *                                      
 * 
 *  m = (st + dr) / 2;
 * 
 *  Lungimea        Nr. Operatii
 *  Intervalului   
 *  n				  0
 *  n/2               1
 *  n/2^2             2
 *  n/2^3             3
 *  ..........
 * 
 *  n/2^k             k  
 * 
 * Alg se opreste in cazul cel mai defavorabil (cand nu s-a gasit v)
 * atunci cand st = dr, deci atunci cand lungimea intervalului devine 1
 * 
 * =>
 *  n/2^k = 1
 *  
 *  2^k = n
 * 
 *  log2(2^k) = log2(n)
 *  
 *  k = log2(n)
 */ 

#include <fstream>
using namespace std;

ifstream fin("bs.in");
ofstream fout("bs.out");

const int Dim = 1000001;
int a[Dim], n;
int val; // valoarea care se cauta

void CitesteDate();
int CautareBinara(int v);

int main()
{
	CitesteDate();
	
	int poz = CautareBinara(val);
	
	if (poz != -1)
		fout << "S-a gasit " << val << " pe pozitia " << poz;
	else
		fout << "Nu s-a gasit " << val;
}

// Cauta v in sirul a[] si returneza pozitia sa sau -1 daca nu s-a gasit v
int CautareBinara(int v)  // O(log2(n))
{
	if (v > a[n]) return -1;
	if (v < a[1]) return -1;
	
	int st = 1, dr = n, m;
	
	while (st <= dr) // cat timp mai exista un interval de cautare
	{
		m = (st + dr) / 2;
		
		if (v == a[m]) // am gasit valoare
			return m;  // returnam pozitia
		
		if (v > a[m])
			st = m + 1; // vom cauta numai intervalul [m + 1, dr]
		else  // sau
			dr = m - 1; // vom cauta numai intervalul [st, m - 1]
	}
	
	return -1; // nu s-a gasit valoarea v
}

void CitesteDate()
{
	fin >> n;
	for (int i = 1; i <= n; ++i)
		fin >> a[i];
	fin >> val;
	
	
}







